﻿define([], function () {
    "use strict";

    var controllerName = "FirmwareUpdate";

    function _getCookie(name) {
        var value = "; " + document.cookie;
        var parts = value.split("; " + name + "=");

        if (parts.length === 2)
            return parts.pop().split(";").shift();
    }

    async function _AJAXSubmit(oFormElement) {
        const formData = new FormData(oFormElement);

        $('#alertResultError').addClass('hidden');
        $('#alertResultError').empty();
        $('#alertResultSuccess').addClass('hidden');
        $('#alertResultSuccess').empty();
        $("#alertInProgress").addClass("hidden");
        $("#alertInProgress").empty();

        try {
            try {
                if ($('#fileSelector').get(0).files.length === 0) {
                    console.log("no files selected");
                    return;
                }

                $('#panelFilesReady').addClass('hidden');
                $('#uploadFileWait').removeClass('hidden');
                $('#btnUpload').prop('disabled', true);
                $('#fileSelector').prop('disabled', true);

                $("#alertInProgress").removeClass("hidden");
                $("#alertInProgress").html("Upload ...");

                const response = await fetch(oFormElement.action, {
                    method: 'POST',
                    headers: { 'RequestVerificationToken': _getCookie('RequestVerificationToken') },
                    body: formData
                });

                if (response.status === 200) {
                    response.json().then(
                        function (value) {
                            $('#alertResultSuccess').html(value.message);
                            $('#alertResultSuccess').removeClass('hidden');
                        })
                        .catch(err => {
                            console.log(err);
                        });
                    $('#fileSelector').val('');

                    _GetResult();
                }
                else {
                    response.json().then(
                        function (value) {
                            $('#alertResultError').html(value.message);
                            $('#alertResultError').removeClass('hidden');
                        })
                        .catch(err => {
                            console.log(err);
                        });
                }
            }
            catch (error) {
                console.error('Error:', error);

                $('#alertResultError').removeClass('hidden');
                $('#alertResultError').html(error);
            }
        }
        finally {
            $('#uploadFileWait').addClass('hidden');
            $('#btnUpload').prop('disabled', false);
            $('#fileSelector').prop('disabled', false);
            $("#alertInProgress").addClass("hidden");
            $("#alertInProgress").empty();
        }
    }

    function _GetResult() {

        $('#uploadFileWait').removeClass('hidden');
        $('#alertResultSuccess').addClass('hidden');
        $('#alertResultError').addClass('hidden');
        $('#btnUpload').prop('disabled', true);
        $('#fileSelector').prop('disabled', true);

        $.ajax({
            url: controllerName + '/GetUploadResult',
            type: 'GET',
            cache: false,
            dataType: "json",
            success: function (firmwareUpdateResult) {
                $('#uploadFileWait').removeClass('hidden');
                $('#btnUpload').prop('disabled', true);
                $('#fileSelector').prop('disabled', true);
                $('#alertResultSuccess').addClass('hidden');
                $('#alertResultError').addClass('hidden');

                if (firmwareUpdateResult) {
                    if (firmwareUpdateResult.message) {
                        $("#alertInProgress").html(firmwareUpdateResult.message);
                        $("#alertInProgress").removeClass("hidden");
                    }

                    if (firmwareUpdateResult.isCompleted !== true) {
                        setTimeout(function () { _GetResult(); }, 5000);

                        return;
                    }

                    $('#uploadFileWait').addClass('hidden');
                    $("#alertInProgress").addClass("hidden");
                    $('#btnUpload').prop('disabled', false);
                    $('#fileSelector').prop('disabled', false);

                    if (firmwareUpdateResult.isSucceeded === true) {
                        if (firmwareUpdateResult.message) {
                            $("#alertResultSuccess").html(firmwareUpdateResult.message);
                        }
                        else {
                            $("#alertResultSuccess").html("success");
                        }
                        $('#alertResultSuccess').removeClass('hidden');
                        $('#alertResultError').addClass('hidden');

                        _PostPrepareCompleted(firmwareUpdateResult);    //Redirect
                    }
                    else {
                        if (firmwareUpdateResult.message) {
                            $("#alertResultError").html(firmwareUpdateResult.message);
                        }
                        else {
                            $("#alertResultError").html("error");
                        }
                        $("#alertResultError").removeClass("hidden");
                        $('#alertResultSuccess').addClass('hidden');
                    }
                }
                else {
                    setTimeout(function () { _GetResult(); }, 5000);
                }
            },
            error: function (jqXHR, textStatus, error) {
                if (jqXHR && jqXHR.responseJSON) {
                    if (jqXHR.responseJSON.message) {
                        $("#alertResultError").html(jqXHR.responseJSON.message);
                        console.log(jqXHR.responseJSON.message);
                    }
                    else {
                        $('#alertResultError').html("error occured");
                    }
                }
                else {
                    $('#alertResultError').html(error);
                }

                $('#uploadFileWait').addClass('hidden');
                $("#alertInProgress").addClass("hidden");
                $('#alertResultSuccess').addClass('hidden');
                $("#alertResultError").removeClass("hidden");
                $('#btnUpload').prop('disabled', false);
                $('#fileSelector').prop('disabled', false);
            },
            complete: function (jqXHR, textStatus) {

            }
        });
    }

    function _PostPrepareCompleted(firmwareUpdateResult) {

        $("#alertResultError").addClass("hidden");
        $('#btnUpload').prop('disabled', false);
        $('#fileSelector').prop('disabled', false);

        $.ajax({
            url: controllerName + '/RedirectStartUpdate',
            type: 'POST',
            headers: { 'RequestVerificationToken': _getCookie('RequestVerificationToken') },
            cache: false,
            dataType: "json",
            data: firmwareUpdateResult,
            success: function (redirectUrl) {
                window.location.replace(redirectUrl);    //Redirect
            },
            error: function (jqXHR, textStatus, error) {

                $('#alertResultSuccess').addClass('hidden');
                $("#alertResultError").removeClass("hidden");

                if (jqXHR && jqXHR.responseJSON) {
                    if (jqXHR.responseJSON.message) {
                        $("#alertResultError").html(jqXHR.responseJSON.message);
                        console.log(jqXHR.responseJSON.message);
                    }
                    else {
                        $('#alertResultError').html("error occured");
                    }
                }
                else {
                    $('#alertResultError').html(error);
                    console.log(error);
                }
            }
        });
    }


    return {
        init: function () {
            $('#uploadForm').submit(function (event) {
                _AJAXSubmit(this);

                return false;
            });
        }
    };
});
